local LibStub = LibStub
local AceAddon = LibStub("AceAddon-3.0")
local AceLocale = LibStub("AceLocale-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local ControlPanel = AceAddon:GetAddon("ControlPanel")
local ControlPanel_Options = AceAddon:GetAddon("ControlPanel_Options")
local L = AceLocale:GetLocale("ControlPanel_Options")
local string_format = string.format
local SetCVar = SetCVar
local get_cvar_number = ControlPanel_Options.get_cvar_number

local function tablefind(array,key)
	local i = 1
	for i = 1,#array do
		if key == array[i] then
			return i
		end
	end
	return i
end

local string_upper = string.upper
local languages = {deDE="deDE",enUS="enUS",esES="esES",esMX="esMX",frFR="frFR",itIT="itIT",koKR="koKR",ptBR="ptBR",ruRU="ruRU",zhCN="zhCN",zhTW="zhTW"}
local allow_global_operations = false
local resolutions = { GetScreenResolutions() }
local graphicsAPIs = { GetGraphicsAPIs() }

local order = 0
local function get_order()
	local temp = order
	order = order +1
	return temp
end
local function SetCVar_Global(key,value)
	if allow_global_operations == true then
		SetCVar(key,value)
	end
	allow_global_operations = false
end
local function SetCVar_Global_RestartGx(key,value)
	if allow_global_operations == true then
		SetCVar(key,value)
		RestartGx()
	end
	allow_global_operations = false
end
local Global =
{
	name = L["Global"],
	desc = L["Requires a client restart."].."\n\n"..L["ControlPanel won't manage these options."],
	type = "group",
	args =
	{
		Allow_Once =
		{
			order = get_order(),
			name = L["Allow once"],
			desc = L["Prevent misoperations on Global Settings"],
			type = "toggle",
			set = function(info,val)
				allow_global_operations = val
			end,
			get  = function(info)
				return allow_global_operations
			end,
			width = "full",
		},
		Display =
		{
			order = get_order(),
			name = L["Display"],
			desc = L["Requires a restart of the game's graphic subsystem"],
			type = "group",
			args =
			{
				gxWindow =
				{
					order = get_order(),
					name = L["Windowed"],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global_RestartGx("gxWindow",val)
					end,
					get = function(info)
						return GetCVarBool("gxWindow")
					end,
				},
				gxMaximize =
				{
					order = get_order(),
					name = L["Fullscreen"],
					type = "toggle",
					set = function(info,val)
						SetCVar_Global_RestartGx("gxMaximize",val)
					end,
					get = function(info)
						return GetCVarBool("gxMaximize")
					end,
				},
				gxResolution =
				{
					order = get_order(),
					name = L["Resolution"],
					desc = L["Higher resolution will result in increased clarity, but this greatly affects performance. Choose a resolution that matches the aspect ratio of your monitor."],
					type = "select",
					values = resolutions,
					set = function(info,val)
						SetCVar_Global_RestartGx("gxResolution",resolutions[val])
					end,
					get = function(info)
						return GetCurrentResolution()
					end,
				},
				gxApi =
				{
					order = get_order(),
					name = L["Graphics API"],
					desc = L["Requires a client restart."],
					type = "select",
					values = graphicsAPIs,
					set = function(info,val)
						SetCVar_Global("gxApi",graphicsAPIs[val])
					end,
					get = function(info)
						return tablefind(graphicsAPIs,GetCVar("gxApi"))
					end,
				},
				violenceLevel =
				{
					order = get_order(),
					name = L["Violence Level"],
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					set = function(info,val)
						SetCVar_Global("violenceLevel",val)
					end,
					get = function(info)
						return get_cvar_number("violenceLevel")
					end,
				},
			}
		},
		Langauges =
		{
			order = get_order(),
			name = L["Langauges"],
			type = "group",
			args =
			{
				portal = 
				{
					order = get_order(),
					name = string_format(L["Portal : %s"],string_upper(GetCVar("portal"))),
					type = "description",
					fontSize = "large",
					width = "full",
				},
				textLocale =
				{
					order = get_order(),
					name = L["Text"],
					type = "select",
					values = languages,
					set = function(info,val)
						if val ~= "zhCN" then
							SetCVar_Global("textLocale",val)
						end
					end,
					get = function(info)
						return GetCVar("textLocale")
					end,
				},
				audioLocale =
				{
					order = get_order(),
					name = L["Audio"],
					type = "select",
					values = languages,
					set = function(info,val)
						if val ~= "zhCN" then
							SetCVar_Global("audioLocale",val)
						end
					end,
					get = function(info)
						return GetCVar("audioLocale")
					end,
				},
				installLocale =
				{
					order = get_order(),
					name = L["Install"],
					type = "select",
					values = languages,
					set = function(info,val)
						if val ~= "zhCN" then
							SetCVar_Global("installLocale",val)
						end
					end,
					get = function(info)
						return GetCVar("installLocale")
					end,
				},
			},
		},
	}
}

ControlPanel_Options:push("Global",Global)
